#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains colors used by Artella widgets
"""

from __future__ import print_function, division, absolute_import

from artella.core import qtutils

if qtutils.QT_AVAILABLE:
    from artella.externals.Qt import QtCore, QtGui

_NUMERALS = '0123456789abcdefABCDEF'
_HEXDEC = {v: int(v, 16) for v in (x + y for x in _NUMERALS for y in _NUMERALS)}


class ArtellaColors(object):

    ARTELLA = '#16a496'
    DEFAULT = ARTELLA
    RED = '#f5222d'
    GREEN = ARTELLA
    BLUE = '#1890ff'
    YELLOW = '#faad14'


def string_is_hex(color_str):
    """
    Returns whether or not given string is a valid hexadecimal color
    :param color_str: str
    :return: bool
    """

    if color_str.startswith('#'):
        color_str = color_str[1:]
    hex_regex1 = QtCore.QRegExp('^[0-9A-F]{3}$', QtCore.Qt.CaseInsensitive)
    hex_regex2 = QtCore.QRegExp('^[0-9A-F]{6}$', QtCore.Qt.CaseInsensitive)
    hex_regex3 = QtCore.QRegExp('^[0-9A-F]{8}$', QtCore.Qt.CaseInsensitive)
    if hex_regex1.exactMatch(color_str) or hex_regex2.exactMatch(color_str) or hex_regex3.exactMatch(color_str):
        return True

    return False


def rgb_from_hex(triplet):
    """
    Returns a RGB triplet from an hexadecimal value
    :param triplet: r,g,b Hexadecimal Color tuple
    """

    if triplet.startswith('#'):
        triplet = triplet[1:]

    if len(triplet) == 3:
        r, g, b = triplet[0] * 2, triplet[1] * 2, triplet[2] * 2
        return tuple([float(int(v, 16)) for v in (r, g, b)])

    return _HEXDEC[triplet[0:2]], _HEXDEC[triplet[2:4]], _HEXDEC[triplet[4:6]]


def from_string(text_color):
    """
    Returns a (int, int, int, int) format color from a string format color
    :param text_color: str, string format color to parse
    :param alpha: int, alpha of the color
    :return: (int, int, int, int)
    """

    a = 255
    if string_is_hex(text_color):
        r, g, b = rgb_from_hex(text_color)
    else:
        try:
            if text_color.startswith('rgba'):
                r, g, b, a = text_color.replace('rgba(', '').replace(')', '').split(',')
            else:
                r, g, b, a = text_color.replace('rgb(', '').replace(')', '').split(',')
        except ValueError:
            if text_color.startswith('rgba'):
                r, g, b = text_color.replace('rgba(', '').replace(')', '').split(',')
            else:
                r, g, b = text_color.replace('rgb(', '').replace(')', '').split(',')

    return QtGui.QColor(int(r), int(g), int(b), int(a))


def generate_color(primary_color, index):
    """
    Generates a new color from the given one and with given index (between 1 and 10)
    https://github.com/phenom-films/dayu_widgets/blob/master/dayu_widgets/utils.py
    :param primary_color: base color (RRGGBB)
    :param index: color step from 1 (light) to 10 (dark)
    :return: out color Color
    """

    hue_step = 2
    saturation_step = 16
    saturation_step2 = 5
    brightness_step1 = 5
    brightness_step2 = 15
    light_color_count = 5
    dark_color_count = 4

    def _get_hue(color, i, is_light):
        h_comp = color.hue()
        if 60 <= h_comp <= 240:
            hue = h_comp - hue_step * i if is_light else h_comp + hue_step * i
        else:
            hue = h_comp + hue_step * i if is_light else h_comp - hue_step * i
        if hue < 0:
            hue += 359
        elif hue >= 359:
            hue -= 359
        return hue / 359.0

    def _get_saturation(color, i, is_light):
        s_comp = color.saturationF() * 100
        if is_light:
            saturation = s_comp - saturation_step * i
        elif i == dark_color_count:
            saturation = s_comp + saturation_step
        else:
            saturation = s_comp + saturation_step2 * i
        saturation = min(100.0, saturation)
        if is_light and i == light_color_count and saturation > 10:
            saturation = 10
        saturation = max(6.0, saturation)
        return round(saturation * 10) / 1000.0

    def _get_value(color, i, is_light):
        v_comp = color.valueF()
        if is_light:
            return min((v_comp * 100 + brightness_step1 * i) / 100, 1.0)
        return max((v_comp * 100 - brightness_step2 * i) / 100, 0.0)

    light = index <= 6
    hsv_color = QtGui.QColor(primary_color) if isinstance(primary_color, str) else primary_color
    index = light_color_count + 1 - index if light else index - light_color_count - 1
    return QtGui.QColor.fromHsvF(
        _get_hue(hsv_color, index, light),
        _get_saturation(hsv_color, index, light),
        _get_value(hsv_color, index, light)
    ).name()


def fade_color(color, alpha):
    """
    Internal function that fades given color
    :param color: QColor
    :param alpha: float
    :return:
    """

    qt_color = QtGui.QColor(color)
    return 'rgba({}, {}, {}, {})'.format(qt_color.red(), qt_color.green(), qt_color.blue(), alpha)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains opacity stack widget implementation
"""

from __future__ import print_function, division, absolute_import

from artella.core import qtutils

if qtutils.QT_AVAILABLE:
    from artella.externals.Qt import QtCore, QtWidgets


def stacked_opacity_animation_mixin(cls):
    """
    Decorators for stacked widget
    When stacked widget index changes, show opacity and position animation for current widget
    :param cls:
    :return:
    """

    if not qtutils.is_stackable(cls):
        return cls

    old_init = cls.__init__

    def _new_init(self, *args, **kwargs):
        old_init(self, *args, **kwargs)
        self._prev_index = 0
        self._to_show_pos_anim = QtCore.QPropertyAnimation()
        self._to_show_pos_anim.setDuration(400)
        self._to_show_pos_anim.setPropertyName(b'pos')
        self._to_show_pos_anim.setEndValue(QtCore.QPoint(0, 0))
        self._to_show_pos_anim.setEasingCurve(QtCore.QEasingCurve.OutCubic)
        self._to_hide_pos_anim = QtCore.QPropertyAnimation()
        self._to_hide_pos_anim.setDuration(400)
        self._to_hide_pos_anim.setPropertyName(b'pos')
        self._to_hide_pos_anim.setEndValue(QtCore.QPoint(0, 0))
        self._to_hide_pos_anim.setEasingCurve(QtCore.QEasingCurve.OutCubic)
        self._opacity_effect = QtWidgets.QGraphicsOpacityEffect()
        self._opacity_anim = QtCore.QPropertyAnimation()
        self._opacity_anim.setDuration(400)
        self._opacity_anim.setEasingCurve(QtCore.QEasingCurve.InCubic)
        self._opacity_anim.setPropertyName(b'opacity')
        self._opacity_anim.setStartValue(0.0)
        self._opacity_anim.setEndValue(1.0)
        self._opacity_anim.setTargetObject(self._opacity_effect)
        self._opacity_anim.finished.connect(self._on_disable_opacity)
        self.currentChanged.connect(self._on_play_anim)

    def _on_play_anim(self, index):
        current_widget = self.widget(index)
        if self._prev_index < index:
            self._to_show_pos_anim.setStartValue(QtCore.QPoint(self.width(), 0))
            self._to_show_pos_anim.setTargetObject(current_widget)
            self._to_show_pos_anim.start()
        else:
            self._to_hide_pos_anim.setStartValue(QtCore.QPoint(-self.width(), 0))
            self._to_hide_pos_anim.setTargetObject(current_widget)
            self._to_hide_pos_anim.start()
        current_widget.setGraphicsEffect(self._opacity_effect)
        current_widget.graphicsEffect().setEnabled(True)
        self._opacity_anim.start()
        self._prev_index = index

    def _on_disable_opacity(self):
        self.currentWidget().graphicsEffect().setEnabled(False)

    setattr(cls, '__init__', _new_init)
    setattr(cls, '_on_play_anim', _on_play_anim)
    setattr(cls, '_on_disable_opacity', _on_disable_opacity)

    return cls


if qtutils.QT_AVAILABLE:
    @stacked_opacity_animation_mixin
    class SlidingOpacityStackedWidget(QtWidgets.QStackedWidget, object):
        """
        Custom stack widget that activates opacity animation when current stack index changes
        """

        def __init__(self, parent=None):
            super(SlidingOpacityStackedWidget, self).__init__(parent)
else:
    class SlidingOpacityStackedWidget(object):
        pass



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains DCC abstract dialog implementation
"""

from __future__ import print_function, division, absolute_import

from artella.core import utils, qtutils, resource
from artella.widgets import theme

if qtutils.QT_AVAILABLE:
    from artella.externals.Qt import QtCore, QtWidgets


class _MetaDialog(type):

    def __call__(cls, *args, **kwargs):
        return BaseDialog


class AbstractDialog(object):

    @utils.abstract
    def get_main_layout(self):
        pass

    @utils.abstract
    def setup_ui(self):
        pass


if not qtutils.QT_AVAILABLE:
    class BaseDialog(AbstractDialog):
        pass
else:
    class BaseDialog(QtWidgets.QDialog, object):

        closed = QtCore.Signal()

        def __init__(self, parent=None, **kwargs):
            if not parent:
                from artella import dcc
                parent = dcc.get_main_window()

            self._use_artella_header = kwargs.pop('use_artella_header', True)

            super(BaseDialog, self).__init__(parent, **kwargs)

            self._pos_anim = QtCore.QPropertyAnimation(self)
            self._pos_anim.setTargetObject(self)
            self._pos_anim.setEasingCurve(QtCore.QEasingCurve.OutCubic)
            self._pos_anim.setDuration(300)
            self._pos_anim.setPropertyName(b'pos')

            self._opacity_anim = QtCore.QPropertyAnimation()
            self._opacity_anim.setTargetObject(self)
            self._opacity_anim.setEasingCurve(QtCore.QEasingCurve.OutCubic)
            self._opacity_anim.setDuration(300)
            self._opacity_anim.setPropertyName(b'windowOpacity')
            self._opacity_anim.setStartValue(0.0)
            self._opacity_anim.setEndValue(1.0)

            self.setup_ui()
            theme.theme().apply(self)

            self._fade_in()

        def get_main_layout(self):
            main_layout = QtWidgets.QVBoxLayout()
            main_layout.setContentsMargins(0, 0, 0, 0)
            main_layout.setSpacing(0)

            return main_layout

        def setup_ui(self):
            self.main_layout = self.get_main_layout()
            self.setLayout(self.main_layout)

            if self._use_artella_header:
                artella_frame = QtWidgets.QFrame()
                artella_frame.setObjectName('artellaFrame')
                artella_frame_layout = QtWidgets.QHBoxLayout()
                artella_frame.setLayout(artella_frame_layout)

                artella_header = QtWidgets.QLabel()
                artella_header_pixmap = resource.pixmap('artella_header')
                artella_header.setPixmap(artella_header_pixmap)
                artella_frame_layout.addStretch()
                artella_frame_layout.addWidget(artella_header)
                artella_frame_layout.addStretch()

                self.main_layout.addWidget(artella_frame)

        def fade_close(self):
            self._fade_out()

        # =================================================================================================================
        # INTERNAL
        # =================================================================================================================

        def _fade_out(self):
            self._opacity_anim.finished.connect(self.close)
            self._pos_anim.setDirection(QtCore.QAbstractAnimation.Backward)
            self._pos_anim.start()
            self._opacity_anim.setDirection(QtCore.QAbstractAnimation.Backward)
            self._opacity_anim.start()

        def _fade_in(self):
            self._pos_anim.start()
            self._opacity_anim.start()


@utils.add_metaclass(_MetaDialog)
class Dialog(AbstractDialog):
    pass



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains Artella label widgets
"""

from __future__ import print_function, division, absolute_import

from artella.core import qtutils

if qtutils.QT_AVAILABLE:
    from artella.externals.Qt import QtCore, QtWidgets


class ArtellaLabelTypes(object):
    SECONDARY = 'secondary'
    WARNING = 'warning'
    DANGER = 'danger'
    H1 = 1
    H2 = 2
    H3 = 3
    H4 = 4


if not qtutils.QT_AVAILABLE:
    class ArtellaLabel(object):
        def __init__(self, *args, **kwargs):
            pass
else:

    class ArtellaLabel(QtWidgets.QLabel, object):
        def __init__(self, text='', parent=None):
            super(ArtellaLabel, self).__init__(text, parent)

            self._actual_text = text
            self._artella_type = ''
            self._artella_underline = False
            self._artella_mark = False
            self._artella_delete = False
            self._artella_strong = False
            self._artella_code = False
            self._artella_level = 0
            self._elide_mode = QtCore.Qt.ElideNone
            self.setProperty('artella_text', text)

            self.setTextInteractionFlags(QtCore.Qt.TextBrowserInteraction | QtCore.Qt.LinksAccessibleByMouse)
            self.setSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Minimum)

        def get_artella_level(self):
            return self._artella_level

        def set_artella_level(self, value):
            self._artella_level = value
            self.style().polish(self)

        def set_artella_underline(self, value):
            self._artella_underline = value
            self.style().polish(self)

        def get_artella_underline(self):
            return self._artella_underline

        def set_artella_delete(self, value):
            self._artella_delete = value
            self.style().polish(self)

        def get_artella_delete(self):
            return self._artella_delete

        def set_artella_strong(self, value):
            self._artella_strong = value
            self.style().polish(self)

        def get_artella_strong(self):
            return self._artella_strong

        def set_artella_mark(self, value):
            self._artella_mark = value
            self.style().polish(self)

        def get_artella_mark(self):
            return self._artella_mark

        def set_artella_code(self, value):
            self._artella_code = value
            self.style().polish(self)

        def get_artella_code(self):
            return self._artella_code

        def get_elide_mode(self):
            return self._elide_mode

        def set_elide_mode(self, value):
            self._elide_mode = value
            self._update_elided_text()

        def get_artella_type(self):
            return self._artella_type

        def set_artella_type(self, value):
            self._artella_type = value
            self.style().polish(self)

        artella_level = QtCore.Property(int, get_artella_level, set_artella_level)
        artella_type = QtCore.Property(str, get_artella_type, set_artella_type)
        artella_underline = QtCore.Property(bool, get_artella_underline, set_artella_underline)
        artella_delete = QtCore.Property(bool, get_artella_delete, set_artella_delete)
        artella_strong = QtCore.Property(bool, get_artella_strong, set_artella_strong)
        artella_mark = QtCore.Property(bool, get_artella_mark, set_artella_mark)
        artella_code = QtCore.Property(bool, get_artella_code, set_artella_code)
        artella_elide_mod = QtCore.Property(QtCore.Qt.TextElideMode, get_artella_code, set_artella_code)

        def minimumSizeHint(self):
            return QtCore.QSize(1, self.fontMetrics().height())

        def text(self):
            return self._actual_text

        def setText(self, text):
            self._actual_text = text
            self._update_elided_text()
            self.setToolTip(text)

        def _update_elided_text(self):
            _font_metrics = self.fontMetrics()
            _elided_text = _font_metrics.elidedText(self._actual_text, self._elide_mode,
                                                    self.width() - 2 * 2)
            super(ArtellaLabel, self).setText(_elided_text)

        def resizeEvent(self, event):
            self._update_elided_text()

        def h1(self):
            self.set_artella_level(ArtellaLabelTypes.H1)
            return self

        def h2(self):
            self.set_artella_level(ArtellaLabelTypes.H2)
            return self

        def h3(self):
            self.set_artella_level(ArtellaLabelTypes.H3)
            return self

        def h4(self):
            self.set_artella_level(ArtellaLabelTypes.H4)
            return self

        def secondary(self):
            self.set_artella_type(ArtellaLabelTypes.SECONDARY)
            return self

        def warning(self):
            self.set_artella_type(ArtellaLabelTypes.WARNING)
            return self

        def danger(self):
            self.set_artella_type(ArtellaLabel.DangerType)
            return self

        def strong(self):
            """Set QLabel with strong style."""
            self.set_artella_strong(True)
            return self

        def mark(self):
            self.set_artella_mark(True)
            return self

        def code(self):
            self.set_artella_code(True)
            return self

        def delete(self):
            self.set_artella_delete(True)
            return self

        def underline(self):
            self.set_artella_underline(True)
            return self

        # def event(self, event):
        #     if event.type() == QEvent.DynamicPropertyChange and event.propertyName() == 'artella_text':
        #         self.setText(self.property('artella_text'))
        #     return super(MLabel, self).event(event)



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains ArtellaDrive Qt related utils classes and functions
"""

from __future__ import print_function, division, absolute_import

import os
import string
import logging
try:
    from cStringIO import StringIO
except ImportError:
    from io import StringIO

from artella.core import utils

QT_AVAILABLE = True
try:
    from artella.externals.Qt import QtCore, QtWidgets, QtGui, __binding__
except ImportError as exc:
    QT_AVAILABLE = False

if QT_AVAILABLE:
    # Some DCCs do not include shiboken, we check all possible locations
    if __binding__ in ['PySide2', 'PyQt5']:
        try:
            import shiboken2 as shiboken
        except ImportError:
            from PySide2 import shiboken2 as shiboken
    else:
        try:
            import shiboken
        except ImportError:
            try:
                from Shiboken import shiboken
            except ImportError:
                try:
                    from PySide import shiboken
                except ImportError:
                    pass

    # If QApplication does not exists we force its creation
    try:
        app = QtWidgets.QApplication.instance()
        if not app:
            QtWidgets.QApplication([])
    except TypeError:
        QT_AVAILABLE = False

DEFAULT_DPI = 96

if utils.is_python3():
    long = int

logger = logging.getLogger('artella')


class StyleTemplate(string.Template):
    delimiter = '@'
    idpattern = r'[_a-z][_a-z0-9]*'


def is_pyqt():
    """
    Returns whether or not current Qt Python binding available is a PyQt one.

    :return: True if the current Qt binding is PyQt; False otherwise.
    :rtype: bool
    """

    return 'PyQt' in __binding__


def is_pyqt4():
    """
    Returns whether or not current Qt Python binding available is PtQt 4.

    :return: True if the current Qt binding is PyQt4; False otherwise.
    :rtype: bool
    """

    return __binding__ == 'PyQt4'


def is_pyqt5():
    """
    Returns whether or not current Qt Python binding available is PtQt 5.

    :return: True if the current Qt binding is PyQt5; False otherwise.
    :rtype: bool
    """

    return __binding__ == 'PyQt5'


def is_pyside():
    """
    Returns whether or not current Qt Python binding available is PySide.

    :return: rue if the current Qt binding is PySide; False otherwise.
    :rtype: bool
    """

    return __binding__ == 'PySide'


def is_pyside2():
    """
    Returns whether or not current Qt Python binding available is PySide 2.

    :return: True if the current Qt binding is PySide2; False otherwise.
    :rtype: bool
    """

    return __binding__ == 'PySide2'


def wrapinstance(ptr, base=None):
    """
    Wraps given object in a Qt object
    :param ptr:
    :param base: QObject, base QWidget class we want to wrap given object
    :return:
    """

    if ptr is None:
        return None

    ptr = long(ptr)
    if 'shiboken' in globals():
        if base is None:
            qObj = shiboken.wrapInstance(long(ptr), QtCore.QObject)
            meta_obj = qObj.metaObject()
            cls = meta_obj.className()
            super_cls = meta_obj.superClass().className()
            if hasattr(QtGui, cls):
                base = getattr(QtGui, cls)
            elif hasattr(QtGui, super_cls):
                base = getattr(QtGui, super_cls)
            else:
                base = QtWidgets.QWidget
        try:
            return shiboken.wrapInstance(long(ptr), base)
        except Exception:
            from PySide.shiboken import wrapInstance
            return wrapInstance(long(ptr), base)
    elif 'sip' in globals():
        base = QtCore.QObject
        return shiboken.wrapinstance(long(ptr), base)
    else:
        logger.error('Failed to wrap object {} ...'.format(ptr))
        return None


def unwrapinstance(obj):
    """
    Unwraps given object from a Qt object
    :param obj: QObject
    :return:
    """

    return long(shiboken.getCppPointer(obj)[0])


def to_qt_object(long_ptr, qobj=None):
    """
    Returns an instance of the Maya UI element as a QWidget
    :param long_ptr:
    :param qobj:
    :return:
    """

    if not qobj:
        qobj = QtWidgets.QWidget

    return wrapinstance(long_ptr, qobj)


def get_active_window():
    """
    Returns current active window
    :return:
    """

    if not QT_AVAILABLE:
        return None

    return QtWidgets.QApplication.activeWindow()


def icon(icon_path, color=None):
    """
    Returns Qt icon instance

    :param icon_path: Path were icon resource is located
    :param color:
    :return: New instance of a Qt icon
    :rtype: QtGui.QIcon
    """

    icon_pixmap = pixmap(icon_path, color=color)
    new_icon = QtGui.QIcon(icon_pixmap)

    return new_icon


def pixmap(pixmap_path, color=None):
    """
    Returns Qt pixmap instance

    :param pixmap_path: Path were pixmap resource is located
    :param color:
    :return: New instance of a Qt pixmap
    :rtype: QtGui.QPixmap
    """

    if color and isinstance(color, str):
        from artella.widgets import color as artella_color
        color = artella_color.from_string(color)

    new_pixmap = QtGui.QPixmap(pixmap_path)

    if not new_pixmap.isNull() and color:
        painter = QtGui.QPainter(new_pixmap)
        painter.setCompositionMode(QtGui.QPainter.CompositionMode_SourceIn)
        painter.setBrush(color)
        painter.setPen(color)
        painter.drawRect(new_pixmap.rect())
        painter.end()

    return new_pixmap


def style(style_path):
    """
    Returns Qt style instance
    :param style_path: Path where style resource is located
    :return: str
    """

    loaded_style = ''
    if not style_path or not os.path.isfile(style_path):
        return loaded_style

    with open(style_path, 'r') as f:
        loaded_style = StyleTemplate(f.read())

    return loaded_style


def show_string_input_dialog(title, label, text='', parent=None):
    """
    Shows a line string input dialog that users can use to input text.

    :param str title: text that is displayed in the title bar of the dialog
    :param str label: text which is shown to the user telling them what kind of text they need to input
    :param str text: default text which is placed n the plain text edit
    :param QWidget parent: optional parent widget for the input text dialog. If not given, current DCC main parent
        window will be used.
    :return: Tuple containing as the first element the text typed by the user and as the second argument a boolean
        that will be True if the user clicked on the Ok button or False if the user cancelled the operation
    :rtype: tuple(str, bool)
    """

    if not QT_AVAILABLE:
        return '', False

    from artella import dcc

    parent = parent if parent else dcc.get_main_window()

    string_dialog = QtWidgets.QInputDialog()
    flags = string_dialog.windowFlags() ^ QtCore.Qt.WindowContextHelpButtonHint | QtCore.Qt.WindowStaysOnTopHint

    typed_string, res = string_dialog.getText(parent, title, label, text=text, flags=flags)

    return typed_string, res


def show_comment_input_dialog(title, label, text='', parent=None):
    """
    Shows a text string input dialog that users can use to input text.

    :param str title: text that is displayed in the title bar of the dialog
    :param str label: text which is shown to the user telling them what kind of text they need to input
    :param str text: default text which is placed in the comment section
    :param QWidget parent: optional parent widget for the input text dialog. If not given, current DCC main parent
        window will be used.
    :return: Tuple containing as the first element the text typed by the user and as the second argument a boolean
        that will be True if the user clicked on the Ok button or False if the user cancelled the operation
    :rtype: tuple(str, bool)
    """

    if not QT_AVAILABLE:
        return '', False

    from artella import dcc

    parent = parent if parent else dcc.get_main_window()

    comment_dialog = QtWidgets.QInputDialog()
    flags = comment_dialog.windowFlags() ^ QtCore.Qt.WindowContextHelpButtonHint | QtCore.Qt.WindowStaysOnTopHint

    if hasattr(QtWidgets.QInputDialog, 'getMultiLineText'):
        comment, res = comment_dialog.getMultiLineText(parent, title, label, text=text, flags=flags)
    else:
        comment, res = comment_dialog.getText(parent, title, label, text, text=text, flags=flags)

    return comment, res


def show_info_message_box(title, text, parent=None):
    """
    Shows an info message box that can be used to show information text to users.

    :param str title: text that is displayed in the title bar of the dialog
    :param str text: default text which is placed n the plain text edit
    :param QWidget parent: optional parent widget for the input text dialog. If not given, current DCC main parent
        window will be used.
    """

    if not QT_AVAILABLE:
        return

    from artella import dcc

    parent = parent if parent else dcc.get_main_window()

    QtWidgets.QMessageBox.information(parent, title, text)


def show_question_message_box(title, text, cancel=True, parent=None):
    """
    Shows a question message box that can be used to show question text to users.

    :param str title: text that is displayed in the title bar of the dialog
    :param str text: default text which is placed n the plain text edit
    :param bool cancel: Whether or not cancel button should appear in question message box
    :param QWidget parent: optional parent widget for the input text dialog. If not given, current DCC main parent
        window will be used.
    :return: True if the user presses the Ok button; False if the user presses the No button; None if the user preses
        the Cancel button
    :rtype: bool or None
    """

    if not QT_AVAILABLE:
        return None

    from artella import dcc

    parent = parent if parent else dcc.get_main_window()

    message_box = QtWidgets.QMessageBox(parent)
    message_box.setWindowTitle(title)
    flags = message_box.windowFlags() ^ QtCore.Qt.WindowContextHelpButtonHint | QtCore.Qt.WindowStaysOnTopHint
    if text:
        message_box.setText(text)
    if cancel:
        message_box.setStandardButtons(
            QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No | QtWidgets.QMessageBox.Cancel)
    else:
        message_box.setStandardButtons(QtWidgets.QMessageBox.Yes | QtWidgets.QMessageBox.No)
    message_box.setWindowFlags(flags)
    result = message_box.exec_()

    if result == QtWidgets.QMessageBox.Yes:
        return True
    elif result == QtWidgets.QMessageBox.No:
        return False
    elif result == QtWidgets.QMessageBox.Cancel:
        return None


def show_warning_message_box(title, text, parent=None):
    """
    Shows a warning message box that can be used to show warning text to users.

    :param str title: text that is displayed in the title bar of the dialog
    :param str text: default text which is placed n the plain text edit
    :param QWidget parent: optional parent widget for the input text dialog. If not given, current DCC main parent
        window will be used.
    """

    if not QT_AVAILABLE:
        return

    from artella import dcc
    from artella.core import resource

    parent = parent if parent else dcc.get_main_window()
    window_icon = resource.icon('artella')

    message_box = QtWidgets.QMessageBox(parent)
    message_box.setWindowTitle(title)
    message_box.setWindowIcon(window_icon)
    message_box.setIcon(message_box.Icon.Warning)
    flags = message_box.windowFlags() ^ QtCore.Qt.WindowContextHelpButtonHint | QtCore.Qt.WindowStaysOnTopHint
    if text:
        message_box.setText(text)
    message_box.setStandardButtons(QtWidgets.QMessageBox.Ok)
    message_box.setWindowFlags(flags)
    message_box.exec_()


def show_error_message_box(title, text, parent=None):
    """
    Shows an error message box that can be used to show critical text to users.

    :param str title: text that is displayed in the title bar of the dialog
    :param str text: default text which is placed n the plain text edit
    :param QWidget parent: optional parent widget for the input text dialog. If not given, current DCC main parent
        window will be used.
    """

    if not QT_AVAILABLE:
        return

    from artella import dcc
    from artella.core import resource

    parent = parent if parent else dcc.get_main_window()
    window_icon = resource.icon('artella')

    message_box = QtWidgets.QMessageBox(parent)
    message_box.setWindowTitle(title)
    message_box.setWindowIcon(window_icon)
    message_box.setIcon(message_box.Icon.Critical)
    flags = message_box.windowFlags() ^ QtCore.Qt.WindowContextHelpButtonHint | QtCore.Qt.WindowStaysOnTopHint
    if text:
        message_box.setText(text)
    message_box.setStandardButtons(QtWidgets.QMessageBox.Ok)
    message_box.setWindowFlags(flags)
    message_box.exec_()


def dpi_multiplier():
    """
    Returns current application DPI multiplier

    :return: float
    """

    return max(1, float(QtWidgets.QApplication.desktop().logicalDpiY()) / float(DEFAULT_DPI))


def dpi_scale(value):
    """
    Resizes by value based on current DPI

    :param int value: value default 2k size in pixels
    :return: size in pixels now DPI monitor is (4k 2k etc)
    :rtype: int
    """

    mult = dpi_multiplier()
    return value * mult


def clear_layout(layout):
    """
    Removes all the widgets added in the given layout
    :param layout: QLayout
    """

    while layout.count():
        child = layout.takeAt(0)
        if child.widget() is not None:
            child.widget().deleteLater()
        elif child.layout() is not None:
            clear_layout(child.layout())


def is_stackable(widget):
    """
    Returns whether or not given widget is stackable
    :param widget: QWidget
    :return: bool
    """

    return issubclass(widget, QtWidgets.QWidget) and hasattr(widget, 'widget') and hasattr(widget, 'currentChanged')
